﻿using Microsoft.AspNetCore.Mvc;
using VeterinaryContracts.BindingModels;
using VeterinaryContracts.BusinessLogicContracts;
using VeterinaryContracts.SearchModels;
using VeterinaryContracts.ViewModels;
using VeterinaryDatabaseImplement.Models;

namespace VeterinaryRestApi.Controllers
{
	[Route("api/[controller]/[action]")]
	[ApiController]
	public class VisitController : Controller
	{
		private readonly ILogger _logger;
		private readonly IVisitLogic _visit;
		public VisitController(ILogger<VisitController> logger, IVisitLogic visit)
		{
			_logger = logger;
			_visit = visit;
		}
		[HttpGet]
		public Tuple<VisitViewModel, List<string>>? GetVisit(int visitId)
		{
			try
			{
				var elem = _visit.ReadElement(new VisitSearchModel { Id = visitId });
				if (elem == null)
					return null;
				return Tuple.Create(elem, elem.VisitPet.Select(x => x.Value.PetName).ToList());
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка получения визита по id={Id}", visitId);
				throw;
			}
		}
		[HttpGet]
		public List<VisitViewModel> GetVisits(int ownerId)
		{
			try
			{
				return _visit.ReadList(new VisitSearchModel { OwnerId = ownerId });
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка получения списка визитов");
				throw;
			}
		}

		[HttpGet]
		public List<VisitViewModel> GetAllVisits()
		{
			try
			{
				return _visit.ReadList(null);
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка получения списка визитов");
				throw;
			}
		}

		[HttpPost]
		public bool CreateVisit(VisitBindingModel model)
		{
			try
			{
				return _visit.Create(model);
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Не удалось создать визит");
				throw;
			}
		}
	}
}
