﻿using Microsoft.AspNetCore.Mvc;
using System.Diagnostics;
using VeterinaryShowDoctorApp.Models;
using VeterinaryContracts.ViewModels;
using VeterinaryContracts.BindingModels;
using System.Text;
using VeterinaryDataModels.Models;
using VeterinaryContracts.SearchModels;
using VeterinaryContracts.StorageContracts;

namespace VeterinaryShowDoctorApp.Controllers
{
	public class HomeController : Controller
	{
		private readonly ILogger<HomeController> _logger;

		public HomeController(ILogger<HomeController> logger)
		{
			_logger = logger;
		}

		public IActionResult Index()
		{
			if (APIDoctor.Doctor == null)
			{
				return Redirect("~/Home/Enter");
			}
			return View(APIDoctor.GetRequest<List<MedicationViewModel>>($"api/medication/getmedications?doctorid={APIDoctor.Doctor.Id}"));
		}

		public IActionResult Privacy()
		{
			if (APIDoctor.Doctor == null)
			{
				return Redirect("~/Home/Enter");
			}
			return View(APIDoctor.Doctor);
		}

		[HttpPost]
		public void Privacy(string login, string password, string fio)
		{
			if (APIDoctor.Doctor == null)
			{
				throw new Exception("Вы как суда попали? Суда вход только авторизованным");
			}
			if (string.IsNullOrEmpty(login) ||
			string.IsNullOrEmpty(password) || string.IsNullOrEmpty(fio))
			{
				throw new Exception("Введите логин, пароль и ФИО");
			}
			APIDoctor.PostRequest("api/doctor/updatedata", new DoctorBindingModel
			{
				Id = APIDoctor.Doctor.Id,
				DoctorFIO = fio,
				Login = login,
				Password = password
			});
			APIDoctor.Doctor.DoctorFIO = fio;
			APIDoctor.Doctor.Login = login;
			APIDoctor.Doctor.Password = password;
			Response.Redirect("Index");
		}

		[HttpGet]
		public IActionResult Enter()
		{
			return View();
		}
		[HttpPost]
		public void Enter(string login, string password)
		{
			if (string.IsNullOrEmpty(login) ||
			string.IsNullOrEmpty(password))
			{
				throw new Exception("Введите логин и пароль");
			}
			APIDoctor.Doctor =
			APIDoctor.GetRequest<DoctorViewModel>($"api/doctor/login?login={login}&password={password}");
			if (APIDoctor.Doctor == null)
			{
				throw new Exception("Неверный логин/пароль");
			}
			Response.Redirect("Index");
		}
		[HttpGet]
		public IActionResult Register()
		{
			return View();
		}
		[HttpPost]
		public void Register(string login, string password, string fio)
		{
			if (string.IsNullOrEmpty(login) ||
			string.IsNullOrEmpty(password) || string.IsNullOrEmpty(fio))
			{
				throw new Exception("Введите логин, пароль и ФИО");
			}
			APIDoctor.PostRequest("api/doctor/register", new
			DoctorBindingModel
			{
				DoctorFIO = fio,
				Login = login,
				Password = password
			});
			Response.Redirect("Enter");
			return;
		}
		// medications
		public IActionResult CreateMedication()
		{
			if (APIDoctor.Doctor == null)
			{
				return Redirect("~/Home/Enter");
			}
			return View();
		}
		[HttpPost]
		public void CreateMedication(string name, int price)
		{
			if (APIDoctor.Doctor == null)
			{
				throw new Exception("Вы как суда попали? Суда вход только авторизованным");
			}
			if (string.IsNullOrEmpty(name) || price <= 0)
			{
				throw new Exception("Ошибка в введенных данных");
			}
			APIDoctor.PostRequest("api/medication/createmedication", new MedicationBindingModel
			{
				MedicationName = name,
				Price = price,
				DoctorId = APIDoctor.Doctor.Id
			});
			Response.Redirect("Index");
		}

		public IActionResult DeleteMedication()
		{
			if (APIDoctor.Doctor == null)
			{
				return Redirect("~/Home/Enter");
			}
			ViewBag.Medications = APIDoctor.GetRequest<List<MedicationViewModel>>($"api/medication/getmedications?doctorid={APIDoctor.Doctor.Id}");
			return View();
		}
		[HttpPost]
		public void DeleteMedication(int medication)
		{
			if (APIDoctor.Doctor == null)
			{
				throw new Exception("Вы как суда попали? Суда вход только авторизованным");
			}
			APIDoctor.PostRequest("api/medication/deletemedication", new MedicationBindingModel
			{
				Id = medication
			});
			Response.Redirect("Index");
		}
		public IActionResult UpdateMedication()
		{
			if (APIDoctor.Doctor == null)
			{
				return Redirect("~/Home/Enter");
			}
			ViewBag.Medications = APIDoctor.GetRequest<List<MedicationViewModel>>($"api/medication/getmedications?doctorid={APIDoctor.Doctor.Id}");
			return View();
		}
		[HttpPost]
		public void UpdateMedication(int medication, string name, int price)
		{
			if (APIDoctor.Doctor == null)
			{
				throw new Exception("Вы как суда попали? Суда вход только авторизованным");
			}
			if (string.IsNullOrEmpty(name) || price <= 0)
			{
				throw new Exception("Ошибка в введенных данных");
			}
			APIDoctor.PostRequest("api/medication/updatemedication", new MedicationBindingModel
			{
				Id = medication,
				MedicationName = name,
				Price = price,
				DoctorId = APIDoctor.Doctor.Id
			});
			Response.Redirect("Index");
		}
		// drugs
		public IActionResult Drugs()
		{
			if (APIDoctor.Doctor == null)
			{
				return Redirect("~/Home/Enter");
			}
			return View(APIDoctor.GetRequest<List<DrugViewModel>>($"api/drug/getdrugs?doctorid={APIDoctor.Doctor.Id}"));

		}

		public IActionResult CreateDrug()
		{
			if (APIDoctor.Doctor == null)
			{
				return Redirect("~/Home/Enter");
			}
			ViewBag.Medications = APIDoctor.GetRequest<List<MedicationViewModel>>($"api/medication/getmedications?doctorid={APIDoctor.Doctor.Id}");

			return View();
		}

		[HttpPost]
		public void CreateDrug(string name, List<int> medications, int count)
		{
			if (APIDoctor.Doctor == null)
			{
				throw new Exception("Вы как суда попали? Суда вход только авторизованным");
			}
			if (string.IsNullOrEmpty(name) || count <=0)
			{
				throw new Exception("Ошибка в введенных данных");
			}
			Dictionary<int, IMedicationModel> a = new Dictionary<int, IMedicationModel>();
			foreach (int medication in medications)
			{
				// фигачит нулл в значение
				a.Add(medication, new MedicationSearchModel { Id = medication } as IMedicationModel);
			}
			APIDoctor.PostRequest("api/drug/createdrug", new DrugBindingModel
			{
				DrugName = name,
				Price = 0,
				DrugMedications = a,
				Count = count,
				DoctorId = APIDoctor.Doctor.Id
			});
			Response.Redirect("Index");
		}

		public IActionResult DeleteDrug()
		{
			if (APIDoctor.Doctor == null)
			{
				return Redirect("~/Home/Enter");
			}
			ViewBag.Drugs = APIDoctor.GetRequest<List<DrugViewModel>>($"api/drug/getdrugs?doctorid={APIDoctor.Doctor.Id}");
			return View();
		}

		[HttpPost]
		public void DeleteDrug(int drug)
		{
			if (APIDoctor.Doctor == null)
			{
				throw new Exception("Вы как суда попали? Суда вход только авторизованным");
			}
			APIDoctor.PostRequest("api/drug/deletedrug", new DrugBindingModel
			{
				Id = drug
			});
			Response.Redirect("Index");
		}

		public IActionResult UpdateDrug()
		{
			if (APIDoctor.Doctor == null)
			{
				return Redirect("~/Home/Enter");
			}
			ViewBag.Drugs = APIDoctor.GetRequest<List<DrugViewModel>>($"api/drug/getdrugs?doctorid={APIDoctor.Doctor.Id}");
			ViewBag.Medications = APIDoctor.GetRequest<List<MedicationViewModel>>($"api/medication/getmedications");
			return View();
		}

		[HttpPost]
		public void UpdateDrug(int drug, string name, List<int> medications, int count)
		{
			if (APIDoctor.Doctor == null)
			{
				throw new Exception("Вы как суда попали? Суда вход только авторизованным");
			}
			double _price=0;
			if (string.IsNullOrEmpty(name) || count <= 0)
			{
				throw new Exception("Ошибка в введенных данных");
			}
			Dictionary<int, IMedicationModel> a = new Dictionary<int, IMedicationModel>();
			foreach (int medication in medications)
			{
				a.Add(medication, new MedicationSearchModel { Id = medication } as IMedicationModel);
			}
			foreach (var elem in a)
			{
				_price += elem.Value.Price;
			}
			APIDoctor.PostRequest("api/drug/updatedrug", new DrugBindingModel
			{
				Id = drug,
				DrugName = name,
				Price = Math.Round(_price, 2),
				DrugMedications = a,
				Count = count
			});
			Response.Redirect("Index");
		}

		[HttpGet]
		public Tuple<DrugViewModel, List<string>>? GetDrug(int drugId)
		{
			if (APIDoctor.Doctor == null)
			{
				throw new Exception("Вы как суда попали? Суда вход только авторизованным");
			}
			var result = APIDoctor.GetRequest<Tuple<DrugViewModel, List<string>>>($"api/drug/getdrug?drugid={drugId}");
			if (result == null)
			{
				return default;
			}

			return result;
		}
		// services
		public IActionResult Services()
		{
			if (APIDoctor.Doctor == null)
			{
				return Redirect("~/Home/Enter");
			}
			return View(APIDoctor.GetRequest<List<ServiceViewModel>>($"api/service/getservices?doctorid={APIDoctor.Doctor.Id}"));

		}
		public IActionResult CreateService()
		{
			if (APIDoctor.Doctor == null)
			{
				return Redirect("~/Home/Enter");
			}
			ViewBag.Medications = APIDoctor.GetRequest<List<MedicationViewModel>>($"api/medication/getmedications?doctorid={APIDoctor.Doctor.Id}");
			ViewBag.Visits = APIDoctor.GetRequest<List<VisitViewModel>>($"api/visit/getallvisits");

			return View();
		}
		[HttpPost]
		public void CreateService(string name, List<int> medications, int visit)
		{
			if (APIDoctor.Doctor == null)
			{
				throw new Exception("Вы как суда попали? Суда вход только авторизованным");
			}
			if (string.IsNullOrEmpty(name))
			{
				throw new Exception("Ошибка в введенных данных");
			}
			Dictionary<int, IMedicationModel> a = new Dictionary<int, IMedicationModel>();
			foreach (int medication in medications)
			{
				a.Add(medication, new MedicationSearchModel { Id = medication } as IMedicationModel);
			}

			APIDoctor.PostRequest("api/service/createservice", new ServiceBindingModel
			{
				ServiceName = name,
				ServiceMedications = a,
				VisitId = visit,
				DoctorId = APIDoctor.Doctor.Id
			});
			Response.Redirect("Index");
		}
		public IActionResult DeleteService()
		{
			if (APIDoctor.Doctor == null)
			{
				return Redirect("~/Home/Enter");
			}
			ViewBag.Services = APIDoctor.GetRequest<List<ServiceViewModel>>($"api/service/getservices?doctorid={APIDoctor.Doctor.Id}");
			return View();
		}
		[HttpPost]
		public void DeleteService(int service)
		{
			if (APIDoctor.Doctor == null)
			{
				throw new Exception("Вы как суда попали? Суда вход только авторизованным");
			}
			APIDoctor.PostRequest("api/service/deleteservice", new ServiceBindingModel
			{
				Id = service
			});
			Response.Redirect("Index");
		}
		public IActionResult UpdateService()
		{
			if (APIDoctor.Doctor == null)
			{
				return Redirect("~/Home/Enter");
			}
			ViewBag.Services = APIDoctor.GetRequest<List<ServiceViewModel>>($"api/service/getservices?doctorid={APIDoctor.Doctor.Id}");
			ViewBag.Medications = APIDoctor.GetRequest<List<MedicationViewModel>>($"api/medication/getmedications");
			return View();
		}
		// возможно нужно прописать визит айди
		[HttpPost]
		public void UpdateService(int service, string name, List<int> medications)
		{
			if (APIDoctor.Doctor == null)
			{
				throw new Exception("Вы как суда попали? Суда вход только авторизованным");
			}
			if (string.IsNullOrEmpty(name))
			{
				throw new Exception("Ошибка в введенных данных");
			}
			Dictionary<int, IMedicationModel> a = new Dictionary<int, IMedicationModel>();
			foreach (int medication in medications)
			{
				a.Add(medication, new MedicationSearchModel { Id = medication } as IMedicationModel);
			}
			APIDoctor.PostRequest("api/service/updateservice", new ServiceBindingModel
			{
				Id = service,
				ServiceName = name,
				ServiceMedications = a
			});
			Response.Redirect("Index");
		}
		[HttpGet]
		public Tuple<ServiceViewModel, List<string>>? GetService(int serviceId)
		{
			if (APIDoctor.Doctor == null)
			{
				throw new Exception("Вы как суда попали? Суда вход только авторизованным");
			}
			var result = APIDoctor.GetRequest<Tuple<ServiceViewModel, List<string>>>($"api/service/getservice?serviceid={serviceId}");
			if (result == null)
			{
				return default;
			}

			return result;
		}

		[ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
		public IActionResult Error()
		{
			return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
		}
		// визиты
		public IActionResult Visits()
		{
			if (APIDoctor.Doctor == null)
			{
				return Redirect("~/Home/Enter");
			}
			return View(APIDoctor.GetRequest<List<VisitViewModel>>($"api/visit/getallvisits"));

		}
		// отчеты
		[HttpGet]
		public IActionResult PurchaseMedicationReport()
		{
			if (APIDoctor.Doctor == null)
			{
				return Redirect("~/Home/Enter");
			}
			ViewBag.Medications = APIDoctor.GetRequest<List<MedicationViewModel>>($"api/medication/getmedications?doctorid={APIDoctor.Doctor.Id}");
			return View();
		}
		[HttpGet]
		public IActionResult Report()
		{
			if (APIDoctor.Doctor == null)
			{
				return Redirect("~/Home/Enter");
			}
			return View();
		}
	}
}